unit donnee;

interface

uses
  Classes,
  outils;

type

  TDonnees = class
  private
    fLesEllipsoide : TList;
    fLesDatum : TList;
    fLesProjection : TList;
    fLesMultiple : TList;
  public
    property LesEllipsoide : TList read fLesEllipsoide;
    property LesDatum : TList read fLesDatum;
    property LesProjection : TList read fLesProjection;
    property LesMultiple : TList read fLesMultiple;
    constructor Create;
    destructor Destroy; override;
    procedure AjoutLigne (Ligne : string);
    procedure LoadLigne (Stream : TStream);
    procedure LoadFromStream (Stream : TStream);
    procedure SaveToStream (Stream : TStream);
    // indique l'lment associ  LeQuel
    function UnEllipsoide(LeQuel : string) : TObject;
    function UnDatum(LeQuel : string) : TObject;
    function UneProjection(LeQuel : string) : TObject;
    // indique si un nom est dj utilis
    function Inconnu(Candidat : string) : boolean;
    // retire l'lment demand, vrifie les dpendances, renvoie False si il y en a
    function RetireEllipsoide(LeQuel : string) : boolean;
    function RetireDatum(LeQuel : string) : boolean;
    // extrait la liste des datums et projections
    procedure ListeSystemes(var LaListe : TStringList);
    // retourne la description associe
    function LaDescription(LeQuel : string) : string;
    // ralise une conversion entre les systmes Source et Dest
    function Conversion(NomSource, NomDest, XSource, YSource : string;
                     IndiceSource : integer; var IndiceDest : integer;
                         var XDest, YDest : string; UniteAngle : TUniteAngle) : boolean;
    function Convergence(NomSource, NomDest, XSource, YSource : string;
                         IndiceSource : integer; 
                         var Declinaison : string; UniteAngle : TUniteAngle) : boolean;
  end;

var
  Donnees : TDonnees;

implementation

uses
  SysUtils,
  ellipsoide, datum, projection, multiple, createur;

constructor TDonnees.Create;
begin
  fLesEllipsoide:=TList.Create;
  fLesDatum:=TList.Create;
  fLesProjection:=TList.Create;
  fLesMultiple:=TList.Create;
  AjoutLigne('Ellipsoide="GRS 80" "GRS 80" 6378137 0.08181919106');
  AjoutLigne('Datum="WGS84" "World Geodesic System" "GRS 80" 0 0 0');
end;

destructor TDonnees.Destroy;
var
  I : integer;
  J : integer;
  UnEllipsoide : TEllipsoide;
  UnDatum : TDatum;
  UnProjection : TProjection;
begin
  for I:=LesMultiple.Count-1 downto 0 do
    TMultiple(LesMultiple[I]).Free;
  LesMultiple.Free;
  for I:=LesProjection.Count-1 downto 0 do
    TProjection(LesProjection[I]).Free;
  LesProjection.Free;
  for I:=LesDatum.Count-1 downto 0 do
    TDatum(LesDatum[I]).Free;
  LesDatum.Free;
  for I:=LesEllipsoide.Count-1 downto 0 do
    TEllipsoide(LesEllipsoide[I]).Free;
  LesEllipsoide.Free;
end;

procedure TDonnees.AjoutLigne (Ligne : string);
var
  Flux : TMemoryStream;
begin
  Flux:=TMemoryStream.Create;
  Flux.Write(Ligne[1],length(Ligne));
  Flux.Position:=0;
  LoadLigne(Flux);
  Flux.Free;
end;

procedure TDonnees.LoadLigne (Stream : TStream);
var
  S : string;
  UnEllipsoide : TEllipsoide;
  UnDatum : TDatum;
  UnProjection : TProjection;
  UnMultiple : TMultiple;
  Inconnu : boolean;
  ParamProj : TStringList;

  procedure VerifieNom(Nom : string);
  // vrifie que le nom n'existe pas dj, sinon gnre une exception
  var
    I : integer;
  begin
    for I:=0 to LesEllipsoide.Count-1 do
      if TEllipsoide(LesEllipsoide[I]).Nom=Nom then
        Raise EConvertError.CreateFmt('%s est dj utilis',[Nom]);
    for I:=0 to LesDatum.Count-1 do
      if TDatum(LesDatum[I]).Nom=Nom then
        Raise EConvertError.CreateFmt('%s est dj utilis',[Nom]);
    for I:=0 to LesProjection.Count-1 do
      if TProjection(LesProjection[I]).Nom=Nom then
        Raise EConvertError.CreateFmt('%s est dj utilis',[Nom]);
    for I:=0 to LesMultiple.Count-1 do
      if TMultiple(LesMultiple[I]).Nom=Nom then
        Raise EConvertError.CreateFmt('%s est dj utilis',[Nom]);
  end;

begin
  ParamProj:=TStringList.Create;
  inconnu:=true;
  S:=DebutLigne(Stream);
  if S='Ellipsoide' then
  begin
    inconnu:=false;
    UnEllipsoide:=TEllipsoide.Create;
    try
      UnEllipsoide.LoadFromStream(Stream);
      VerifieNom(UnEllipsoide.Nom);
      LesEllipsoide.Add(UnEllipsoide);
    except
      UnEllipsoide.Free;
    end;
  end;
  if S='Datum' then
  begin
    inconnu:=false;
    UnDatum:=TDatum.Create;
    try
      UnDatum.LoadFromStream(Stream, self);
      VerifieNom(UnDatum.Nom);
      LesDatum.Add(UnDatum);
    except
      UnDatum.Free;
    end;
  end;
  if pos('Projection',S)=1 then
  begin
    UnProjection:=nil;
    try
      inconnu:=false;
      DetailParam(S,ParamProj);
      if ParamProj.Count<>2 then
      begin
        FinLigne(Stream);
        raise EConvertError.CreateFmt('Dfinission de projection ''%s'' incorrecte',[S]);
      end;
      if ParamProj[0]<>'Projection' then
      begin
        FinLigne(Stream);
        Raise EConvertError.CreateFmt('Donne ''%s'' de nature inconnue',[S]);
      end;
      UnProjection:=CreateProjection(ParamProj[1]);
      if UnProjection=nil then
      begin
        FinLigne(Stream);
        Raise EConvertError.CreateFmt('Projection %s inconnue',[S]);
      end;
      UnProjection.LoadFromStream(Stream);
      VerifieNom(UnProjection.Nom);
      LesProjection.Add(UnProjection);
    except
      UnProjection.Free;
    end;
  end;
  if pos('Multiple',S)=1 then
  begin
    UnMultiple:=nil;
    try
      inconnu:=false;
      DetailParam(S,ParamProj);
      if ParamProj.Count<>2 then
      begin
        FinLigne(Stream);
        raise EConvertError.CreateFmt('Dfinission de Multiple ''%s'' incorrecte',[S]);
      end;
      if ParamProj[0]<>'Multiple' then
      begin
        FinLigne(Stream);
        Raise EConvertError.CreateFmt('Donne ''%s'' de nature inconnue',[S]);
      end;
      UnMultiple:=CreateMultiple(ParamProj[1]);
      if UnMultiple=nil then
      begin
        FinLigne(Stream);
        Raise EConvertError.CreateFmt('Multiple %s inconnue',[S]);
      end;
      UnMultiple.LoadFromStream(Stream);
      VerifieNom(UnMultiple.Nom);
      LesMultiple.Add(UnMultiple);
    except
      UnMultiple.Free;
    end;
  end;
  if inconnu then
    try
      FinLigne(Stream);
      Raise EConvertError.CreateFmt('Donne %s de nature inconnue',[S]);
    except
    end;
  ParamProj.Free;
end;

procedure TDonnees.LoadFromStream (Stream : TStream);
begin
  assert(DecimalSeparator='.');
  Stream.Position:=0;
  while Stream.Position<Stream.Size do
    LoadLigne(Stream);
end;

procedure TDonnees.SaveToStream (Stream : TStream);
var
  I : integer;
begin
  assert(DecimalSeparator='.');
  // on ne sauve pas le premier ellipsode qui est GRS80
  for I:=1 to LesEllipsoide.Count-1 do
    TEllipsoide(LesEllipsoide[I]).SaveToStream(Stream);
  // on ne sauve pas le premier datum qui WGS84  
  for I:=1 to LesDatum.Count-1 do
    TDatum(LesDatum[I]).SaveToStream(Stream);
  for I:=0 to LesProjection.Count-1 do
    TProjection(LesProjection[I]).SaveToStream(Stream);
  for I:=0 to LesMultiple.Count-1 do
    TMultiple(LesMultiple[I]).SaveToStream(Stream);
end;

procedure TDonnees.ListeSystemes(var LaListe : TStringList);
// extrait la liste des datums et projections
var
  I : integer;
begin
  LaListe.Clear;
  for I:=0 to LesDatum.Count-1 do
    LaListe.Add(TDatum(LesDatum[I]).Nom);
  for I:=0 to LesProjection.Count-1 do
    LaListe.Add(TProjection(LesProjection[I]).Nom);
  for I:=0 to LesMultiple.Count-1 do
    LaListe.Add(TMultiple(LesMultiple[I]).Nom);
end;

function TDonnees.Conversion(NomSource, NomDest, XSource, YSource : string;
                     IndiceSource : integer; var IndiceDest : integer;
                     var XDest, YDest : string; UniteAngle : TUniteAngle) : boolean;
// ralise une conversion entre les systmes Source et Dest
// NomSource et NomDest sont les Nom identifiants les systmes source et destination
// X et Y sont les coordonnes
// s'il s'agit d'une projection, ce sont des mtres
// IndiceSource et IndiceDestination indiquent les numros de fuseau
// s'il s'agit d'une projection, ce sont des mtres
// IndiceSource et IndiceDestination sont indfinis et inutiliss
// s'il s'agit d'un Datum, X indique la Latitude et Y la Longitude
// IndiceSource=0 pour une origine  Greenwich, =1 pour Paris
// UniteAngle indique l'unit d'angle  utiliser pour la destination
var
  Lat, Long, Xproj, Yproj, Xcart, Ycart, Zcart : real;
  I : integer;
  Echec : boolean;
begin
// conversion vers coordonnes cartsiennes
// depuis datum
  Echec:=true;
  try
    for I:=0 to LesDatum.Count-1 do
     with TDatum(LesDatum[I]) do
      if NomSource=Nom then
      begin
        try
          Lat:=StrToAngle(XSource);
        except
          raise EConvertError.Create('Latitude de dpart incorrecte, exemple 224''17.036''''');
        end;
        try
          Long:=StrToAngle(YSource);
        except
          raise EConvertError.Create('Longitude de dpart incorrecte, exemple 224''17.036''''');
        end;
        VersCartesien(Lat, Long, (IndiceSource=1), Xcart,Ycart, Zcart);
        Echec:=false;
      end;
  // ou depuis projection
    for I:=0 to LesProjection.Count-1 do
     with TProjection(LesProjection[I]) do
      if NomSource=Nom then
      begin
        try
          Xproj:=StrToFloat(XSource);
        except
          raise EConvertError.Create('Easting de dpart incorrecte, exemple 231500');
        end;
        try
          Yproj:=StrToFloat(YSource);
        except
          raise EConvertError.Create('Northing de dpart incorrecte, exemple 231500');
        end;
        VersLatLong(Xproj, Yproj, Lat, Long);
        Datum.VersCartesien(Lat, Long, false, Xcart, Ycart, Zcart);
        Echec:=false;
      end;
  // ou depuis projection multiple
    for I:=0 to LesMultiple.Count-1 do
     with TMultiple(LesMultiple[I]) do
      if NomSource=Nom then
      begin
        try
          Xproj:=StrToFloat(XSource);
        except
          raise EConvertError.Create('Easting de dpart incorrecte, exemple 231500');
        end;
        try
          Yproj:=StrToFloat(YSource);
        except
          raise EConvertError.Create('Northing de dpart incorrecte, exemple 231500');
        end;
        VersLatLong(Xproj, Yproj, IndiceSource, Lat, Long);
        Datum.VersCartesien(Lat, Long, false, Xcart, Ycart, Zcart);
        Echec:=false;
      end;
  finally
  end;
  if not Echec then
  try
    Echec:=true;
  // conversion inverse depuis les coordonnes cartsiennes
    for I:=0 to LesDatum.Count-1 do
     with TDatum(LesDatum[I]) do
      if NomDest=Nom then
      begin
        VersLatLong(Xcart, Ycart, Zcart, (IndiceDest=1), Lat, Long);
        XDest:=AngleToStr(Lat,UniteAngle);
        YDest:=AngleToStr(Long,UniteAngle);
        Echec:=false;
      end;
  // ou vers projection
    for I:=0 to LesProjection.Count-1 do
     with TProjection(LesProjection[I]) do
      if NomDest=Nom then
      begin
        Datum.VersLatLong(Xcart, Ycart, Zcart, false, Lat, Long);
        VersGrille(Lat, Long, Xproj, Yproj);
        XDest:=Format('%.3f',[Xproj]);
        YDest:=Format('%.3f',[Yproj]);
        Echec:=false;
      end;
  // ou vers projection multiple
    for I:=0 to LesMultiple.Count-1 do
     with TMultiple(LesMultiple[I]) do
      if NomDest=Nom then
      begin
        Datum.VersLatLong(Xcart, Ycart, Zcart, false, Lat, Long);
        VersGrille(Lat, Long, Xproj, Yproj, IndiceDest);
        XDest:=Format('%.3f',[Xproj]);
        YDest:=Format('%.3f',[Yproj]);
        Echec:=false;
      end;
  finally
  end;
  result:=not Echec;
end;

function TDonnees.Convergence(NomSource, NomDest, XSource, YSource : string;
                             IndiceSource : integer;
                             var Declinaison : string; UniteAngle : TUniteAngle) : boolean;
// calcul de la convergence au point XSource, YSource
// faute de formule exacte, le calcul est ralis par essai autour du point choisi
// on se dcale de 100m en direction du Nord et on calcule l'angle rsultant
type
  Doublet = array[0..1] of real;
var
  Lat, Long, Xproj, Yproj, Xcart, Ycart, Zcart : Doublet;
  I, J, IndiceDest : integer;
  Echec : boolean;

  function Angle_Doublet(X, Y : Doublet) : real;
  begin
    result:=-arctan((X[1]-X[0])/(Y[1]-Y[0]));
  end;

begin
// conversion vers coordonnes cartsiennes
// depuis datum
  Echec:=true;
  try
    for I:=0 to LesDatum.Count-1 do
     with TDatum(LesDatum[I]) do
      if NomSource=Nom then
      begin
        try
          Lat[0]:=StrToAngle(XSource);
          Lat[1]:=Lat[0]+1.5e-5;
        except
          raise EConvertError.Create('Latitude de dpart incorrecte, exemple 224''17.036''''');
        end;
        try
          Long[0]:=StrToAngle(YSource);
          Long[1]:=Long[0];
        except
          raise EConvertError.Create('Longitude de dpart incorrecte, exemple 224''17.036''''');
        end;
        for J:=0 to 1 do
          VersCartesien(Lat[J], Long[J], (IndiceSource=1), Xcart[J],Ycart[J], Zcart[J]);
        Echec:=false;
      end;
  // ou depuis projection
    for I:=0 to LesProjection.Count-1 do
     with TProjection(LesProjection[I]) do
      if NomSource=Nom then
      begin
        try
          Xproj[0]:=StrToFloat(XSource);
          Xproj[1]:=XProj[0];
        except
          raise EConvertError.Create('Easting de dpart incorrecte, exemple 231500');
        end;
        try
          Yproj[0]:=StrToFloat(YSource);
          Yproj[1]:=Yproj[0]+100;
        except
          raise EConvertError.Create('Northing de dpart incorrecte, exemple 231500');
        end;
        for J:=0 to 1 do
        begin
          VersLatLong(Xproj[J], Yproj[J], Lat[J], Long[J]);
          Datum.VersCartesien(Lat[J], Long[J], false, Xcart[J], Ycart[J], Zcart[J]);
        end;
        Echec:=false;
      end;
  // ou depuis projection multiple
    for I:=0 to LesMultiple.Count-1 do
     with TMultiple(LesMultiple[I]) do
      if NomSource=Nom then
      begin
        try
          Xproj[0]:=StrToFloat(XSource);
          Xproj[1]:=XProj[0];
        except
          raise EConvertError.Create('Easting de dpart incorrecte, exemple 231500');
        end;
        try
          Yproj[0]:=StrToFloat(YSource);
          Yproj[1]:=Yproj[0]+100;
        except
          raise EConvertError.Create('Northing de dpart incorrecte, exemple 231500');
        end;
        for J:=0 to 1 do
        begin
          VersLatLong(Xproj[J], Yproj[J], IndiceSource, Lat[J], Long[J]);
          Datum.VersCartesien(Lat[J], Long[J], false, Xcart[J], Ycart[J], Zcart[J]);
        end;
        Echec:=false;
      end;
  finally
  end;
  if not Echec then
  try
    Echec:=true;
  // conversion inverse depuis les coordonnes cartsiennes
    for I:=0 to LesDatum.Count-1 do
     with TDatum(LesDatum[I]) do
      if NomDest=Nom then
      begin
        for J:=0 to 1 do
          VersLatLong(Xcart[J], Ycart[J], Zcart[J], (IndiceDest=1), Lat[J], Long[J]);
        Declinaison:=AngleToStr(Angle_Doublet(Long,Lat),UniteAngle);
        Echec:=false;
      end;
  // ou vers projection
    for I:=0 to LesProjection.Count-1 do
     with TProjection(LesProjection[I]) do
      if NomDest=Nom then
      begin
        for J:=0 to 1 do
        begin
          Datum.VersLatLong(Xcart[J], Ycart[J], Zcart[J], false, Lat[J], Long[J]);
          VersGrille(Lat[J], Long[J], Xproj[J], Yproj[J]);
        end;
        Declinaison:=AngleToStr(Angle_Doublet(XProj,YProj),UniteAngle);
        Echec:=false;
      end;
  // ou vers projection multiple
    for I:=0 to LesMultiple.Count-1 do
     with TMultiple(LesMultiple[I]) do
      if NomDest=Nom then
      begin
        for J:=0 to 1 do
        begin
          Datum.VersLatLong(Xcart[J], Ycart[J], Zcart[J], false, Lat[J], Long[J]);
          VersGrille(Lat[J], Long[J], Xproj[J], Yproj[J], IndiceDest);
        end;
        Declinaison:=AngleToStr(Angle_Doublet(XProj,YProj),UniteAngle);
        Echec:=false;
      end;
  finally
  end;
  result:=not Echec;
end;

function TDonnees.UnEllipsoide(LeQuel : string) : TObject;
var
  I : integer;
begin
  result:=nil;
  for I:=0 to LesEllipsoide.Count-1 do
    if LeQuel=TEllipsoide(LesEllipsoide[I]).Nom then
      result:=LesEllipsoide[I];
end;

function TDonnees.UnDatum(LeQuel : string) : TObject;
var
  I : integer;
begin
  result:=nil;
  for I:=0 to LesDatum.Count-1 do
    if LeQuel=TDatum(LesDatum[I]).Nom then
      result:=LesDatum[I];
end;

function TDonnees.UneProjection(LeQuel : string) : TObject;
var
  I : integer;
begin
  result:=nil;
  for I:=0 to LesProjection.Count-1 do
    if LeQuel=TProjection(LesProjection[I]).Nom then
      result:=LesProjection[I];
  if result=nil then
    for I:=0 to LesMultiple.Count-1 do
      if LeQuel=TMultiple(LesMultiple[I]).Nom then
        result:=LesMultiple[I];
end;

function TDonnees.RetireEllipsoide(LeQuel : string) : boolean;
// retire l'llipsode demand, vrifie les dpendances, renvoie False si il y en a
var
  Candidat : TEllipsoide;
  I : integer;
begin
  Candidat:=TEllipsoide(UnEllipsoide(LeQuel));
  I:=0;
  while (I<LesDatum.Count) and (Candidat<>TDatum(LesDatum[I]).Ellipsoide) do
    inc(I);
  result:=(I=LesDatum.Count);
  if result then
  begin
    LesEllipsoide.Remove(Candidat);
    Candidat.Free;
  end;
end;

function TDonnees.RetireDatum(LeQuel : string) : boolean;
// retire le datum demand, vrifie les dpendances, renvoie False si il y en a
var
  Candidat : TDatum;
  I : integer;
begin
  Candidat:=TDatum(UnDatum(LeQuel));
  I:=0;
  while (I<LesProjection.Count) and (Candidat<>TProjection(LesProjection[I]).Datum) do
    inc(I);
  result:=(I=LesProjection.Count);
  if result then
  begin
    I:=0;
    while (I<LesMultiple.Count) and (Candidat<>TMultiple(LesMultiple[I]).Datum) do
      inc(I);
    result:=(I=LesMultiple.Count);
    if result then
    begin
      LesDatum.Remove(Candidat);
      Candidat.Free;
    end;  
  end;
end;


function TDonnees.LaDescription(LeQuel : string) : string;
// retourne la description associe
var
  I : integer;
begin
  result:='';
  for I:=0 to LesDatum.Count-1 do
    if LeQuel=TDatum(LesDatum[I]).Nom then
      result:=TDatum(LesDatum[I]).Description;
  for I:=0 to LesProjection.Count-1 do
    if LeQuel=TProjection(LesProjection[I]).Nom then
      result:=TProjection(LesProjection[I]).Description;
  for I:=0 to LesMultiple.Count-1 do
    if LeQuel=TMultiple(LesMultiple[I]).Nom then
      result:=TMultiple(LesMultiple[I]).Description;
end;

function TDonnees.Inconnu(Candidat : string) : boolean;
// indique si un nom est dj utilis
var
  I : integer;
begin
  result:=true;
  for I:=0 to LesDatum.Count-1 do
    if TDatum(LesDatum[I]).Nom=Candidat then
      result:=false;
  for I:=0 to LesProjection.Count-1 do
    if TProjection(LesProjection[I]).Nom=Candidat then
      result:=false;
  for I:=0 to LesMultiple.Count-1 do
    if TMultiple(LesMultiple[I]).Nom=Candidat then
      result:=false;
end;

end.
