unit hotine;

// cas de la projection Oblique Mercator - Hotine

// documentation de rfrence
//
// NOAA Manual NOS NGS 5
// State Plane Coordinate System of 1983
//
// http://www.ngs.noaa.gov/PUBS_LIB/ManualNOSNGS5.pdf

// note : les longitudes sont de signe oppos (positif vers l'ouest) dans le
// document de rfrence par rapport au reste du logiciel (positif vers l'est)
// d'o des changements de signe dans les quations

// note 2 : non teste sur un exemple concret

interface

uses
  projection;

type

  THotine = class(TProjection)
  private
    fLandaC : string; // longitude de l'origine par rapport au mridien d'origine du Datum
    fLandaCReal : real; // valeur en radian de la longitude prcdente
    fPhiC : string; // latitude de l'origine
    fPhiCReal : real; // valeur en radian de la latitude prcdente
    fAlphaC : string; // obliquit de l'indicatrice
    fAlphaCReal : real; // valeur en radian de l'obliquit
    fkC : real; // facteur d'chelle au point d'origine
    fXC, fYC : real; // coordonnes en projection du point d'origine
    procedure Preparation(var F0, F2, F4, F6, B, C, D, Landa0, F, G, I : real);
  protected
    // real
    function GetNbReal : integer; override;
    function GetParamReal(index : integer) : real; override;
    procedure SetParamReal(index : integer; Value : real); override;
    function GetNom_ParamReal(index : integer) : string; override;
    // integer
    function GetNbInteger : integer; override;
    function GetParamInteger(index : integer) : Integer; override;
    procedure SetParamInteger(index : integer; Value : Integer); override;
    function GetNom_ParamInteger(index : integer) : string; override;
    // Angle
    function GetNbAngle : integer; override;
    function GetParamAngle(index : integer) : string; override;
    procedure SetParamAngle(index : integer; Value : string); override;
    function GetNom_ParamAngle(index : integer) : string; override;
    // Boolean
    function GetNbBoolean : integer; override;
    function GetParamBoolean(index : integer) : Boolean; override;
    procedure SetParamBoolean(index : integer; Value : Boolean); override;
    function GetNom_ParamBoolean(index : integer) : string; override;

    function GetNature : string; override;
  public
    procedure VersGrille(Lat, Long : real; var X, Y : real); override;
    procedure VersLatLong(X, Y : real; var Lat, Long : real); override;
  end;

implementation

uses
  SysUtils, Math, 
  outils, ign;


function THotine.GetNature : string;
begin
  result:=tpHotine;
end;

function THotine.GetNbReal : integer;
begin
  result:=3;
end;

function THotine.GetParamReal(index : integer) : real;
begin
  case index of
    0 : result:=fkC;
    1 : result:=fXC;
    2 : result:=fYC;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

procedure THotine.SetParamReal(index : integer; Value : real);
begin
  case Index of
    0 : fkC:=Value;
    1 : fXC:=Value;
    2 : fYC:=Value;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;
end;

function THotine.GetNom_ParamReal(index : integer) : string;
begin
  case index of
    0 : result:='kC';
    1 : result:='XC';
    2 : result:='YC';
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

function THotine.GetNbInteger : integer;
begin
  result:=0;
end;

function THotine.GetParamInteger(index : integer) : Integer;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure THotine.SetParamInteger(index : integer; Value : Integer);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function THotine.GetNom_ParamInteger(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function THotine.GetNbAngle : integer;
begin
  result:=3;
end;

function THotine.GetParamAngle(index : integer) : string;
begin
  case index of
    0 : result:=fLandaC;
    1 : result:=fPhiC;
    2 : result:=fAlphaC;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;  
end;

procedure THotine.SetParamAngle(index : integer; Value : string);
begin
  case index of
    0 : begin
      try
        fLandaC:=Value;
        fLandaCreal:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
    1 : begin
      try
        fPhiC:=Value;
        fPhiCreal:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
    2 : begin
      try
        fAlphaC:=Value;
        fAlphaCreal:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;
end;

function THotine.GetNom_ParamAngle(index : integer) : string;
begin
  case index of
    0 : result:='LandaC';
    1 : result:='PhiC';
    2 : result:='AlphaC';
  else
    begin
      result:='';
      raise ERangeError.Create('Demande de paramtre hors-limites');
    end;
  end;
end;

function THotine.GetNbBoolean : integer;
begin
  result:=0;
end;

function THotine.GetParamBoolean(index : integer) : Boolean;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure THotine.SetParamBoolean(index : integer; Value : Boolean);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function THotine.GetNom_ParamBoolean(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure THotine.Preparation(var F0, F2, F4, F6, B, C, D, Landa0, F, G, I : real);
var
  e2, ep2 : real;
  c2, c4, c6, c8 : real;
  Wc, A, Qc, sinPhiC : real;
begin
  e2:=sqr(Datum.Ellipsoide.e);
  ep2:=e2/(1-e2);

  c2:=e2/2+5/24*sqr(e2)+Intpower(e2,3)/12+13/360*IntPower(e2,4);
  c4:=7/48*sqr(e2)+29/240*IntPower(e2,3)+811/11520*IntPower(e2,4);
  c6:=7/120*IntPower(e2,3)+81/1120*IntPower(e2,4);
  c8:=4279/161280*IntPower(e2,4);
  F0:=2*(c2-2*c4+3*c6-4*c8);
  F2:=8*(c4-4*c6+10*c8);
  F4:=32*(c6-6*c8);
  F6:=128*c8;

  B:=sqrt(1+ep2*IntPower(cos(fPhiCReal),4));
  Wc:=sqrt(1-e2*sqr(sin(fPhiCReal)));
  A:=Datum.Ellipsoide.a*B*sqrt(1-e2)/sqr(Wc);
  sinPhiC:=sin(fPhiCReal);
  Qc:=0.5*(ln((1+sinPhiC)/(1-sinPhiC))-
      Datum.Ellipsoide.e*ln((1+Datum.Ellipsoide.e*sinPhiC)/(1-Datum.Ellipsoide.e*sinPhiC)));
  C:=ArcCosh(B*sqrt(1-e2)/(Wc*cos(fPhiCReal)))-B*Qc;
  D:=fkc*A/B;
  F:=Datum.Ellipsoide.a*sin(fAlphaCReal)*cos(fPhiCReal)/(A*Wc); // sin(Alpha0)
  G:=sqrt(1-sqr(F)); // cos(Alpha0) adaptation personnelle
  Landa0:=-fLandaCReal + arcsin(F*sinh(B*Qc+C)/G)/B;
  I:=fkc*A/Datum.Ellipsoide.a;
end;


procedure THotine.VersGrille(Lat, Long : real; var X, Y : real);
var
  F0, F2, F4, F6, B, C, D, Landa0, F, G, I : real;
  L, Q, J, K, u, v : real;
  sinPhi : real;
begin
  Preparation(F0, F2, F4, F6, B, C, D, Landa0, F, G, I);
  L:=(-Long-Landa0)*B;
  sinPhi:=sin(Lat);
  Q:=0.5*(ln((1+sinPhi)/(1-sinPhi))-
     Datum.Ellipsoide.e*ln((1+Datum.Ellipsoide.e*sinPhi)/(1-Datum.Ellipsoide.e*sinPhi)));
  J:=sinh(B*Q+C);
  K:=cosh(B*Q+C);

  u:=D*arctan((J*G-F*sin(L))/cos(L));
  v:=D/2*ln((K-F*J-G*sin(L))/(K+F*J+G*sin(L)));
  Y:=u*cos(fAlphaCReal)-v*sin(fAlphaCReal)+fYc;
  X:=u*sin(fAlphaCReal)+v*cos(fAlphaCReal)+fXc;
end;  

procedure THotine.VersLatLong(X, Y : real; var Lat, Long : real);
var
  F0, F2, F4, F6, B, C, D, Landa0, F, G, I : real;
  Q, R, S, T, Khi, u, v : real;
  cosKhi : real;
begin
  Preparation(F0, F2, F4, F6, B, C, D, Landa0, F, G, I);
  u:=(X-fXc)*sin(fAlphaCReal)+(Y-fYc)*cos(fAlphaCReal);
  v:=(X-fXc)*cos(fAlphaCReal)-(Y-fYc)*sin(fAlphaCReal);
  R:=sinh(v/D);
  S:=cosh(v/D);
  T:=sin(u/D);
  Q:=(0.5*ln((S-R*F+G*T)/(S+R*F-G*T))-C)/B;
  Khi:=2*arctan((exp(Q)-1)/(exp(Q)+1));
  cosKhi:=cos(Khi);
  Lat:=Khi+sin(Khi)*cosKhi*(F0+F2*sqr(cosKhi)+F4*IntPower(cosKhi,4)+F6*IntPower(cosKhi,6));
  Long:=-(Landa0-arctan((R*G+T*F)/cos(u/D))/B);
end;


end.
