unit laborde_oblic;

// cas de la projection Gauss-Laborde Oblique

// documentation de rfrence
// D'aprs le code C de D. Steinwand/EROS 4/94 lui-mme driv du code GWBASIC
// de John P. Snyder

// Rsultats vrifis avec
//
// Trait des projections des cartes gographiques
//  l'usage  des cartographes et des godsiens
// L. Driencourt et J. Laborde
// quatrime fascicule
// Thorie de la reprsentation conforme
// Emploi des projections rigoureusement conformes en godsie
//
// Notice d'emploi de la projection Mercator Oblique
// Foiben-Taosarintanin' I Madagasikara (institut gographique malgache)
// Solonavalona Andriamihaja

interface

uses
  projection;

type

  TLabordeOblic = class(TProjection)
  private
    fLanda0 : string; // longitude de l'origine par rapport au mridien d'origine du Datum
    fLanda0Real : real; // valeur en radian de la longitude prcdente
    fPhi0 : string; // latitude de l'origine
    fPhi0Real : real; // valeur en radian de la latitude prcdente
    fTheta0 : string; // obliquit de l'indicatrice
    fTheta0Real : real; // valeur en radian de l'obliquit prcdente
    fk0 : real; // facteur d'chelle  l'origine
    fX0, fY0 : real; // coordonnes en projection du point d'origine

    // les champs suivants sont utiliss en interne pour les calculs
    r : real; // rayon de la sphre de courbure
//    sph0 : real;
    phisl, alpha, fchi0, acon, bcon : real;
    fReady : boolean; // indique si les champs prcdents ont t calculs

    procedure Prepare;
  protected
    // real
    function GetNbReal : integer; override;
    function GetParamReal(index : integer) : real; override;
    procedure SetParamReal(index : integer; Value : real); override;
    function GetNom_ParamReal(index : integer) : string; override;
    // integer
    function GetNbInteger : integer; override;
    function GetParamInteger(index : integer) : Integer; override;
    procedure SetParamInteger(index : integer; Value : Integer); override;
    function GetNom_ParamInteger(index : integer) : string; override;
    // Angle
    function GetNbAngle : integer; override;
    function GetParamAngle(index : integer) : string; override;
    procedure SetParamAngle(index : integer; Value : string); override;
    function GetNom_ParamAngle(index : integer) : string; override;
    // Boolean
    function GetNbBoolean : integer; override;
    function GetParamBoolean(index : integer) : Boolean; override;
    procedure SetParamBoolean(index : integer; Value : Boolean); override;
    function GetNom_ParamBoolean(index : integer) : string; override;

    function GetNature : string; override;
  public
    constructor Create; override;
    procedure VersGrille(Lat, Long : real; var X, Y : real); override;
    procedure VersLatLong(X, Y : real; var Lat, Long : real); override;
  end;

implementation

uses
  SysUtils,
  outils, ign, math;

constructor TLabordeOblic.Create;
begin
  fReady:=false;
  inherited Create;
end;  

function TLabordeOblic.GetNature : string;
begin
  result:=tpGaussLabordeOblique;
end;

function TLabordeOblic.GetNbReal : integer;
begin
  result:=3;
end;

function TLabordeOblic.GetParamReal(index : integer) : real;
begin
  case index of
    0 : result:=fk0;
    1 : result:=fX0;
    2 : result:=fY0;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

procedure TLabordeOblic.SetParamReal(index : integer; Value : real);
begin
  case Index of
    0 : fk0:=Value;
    1 : fX0:=Value;
    2 : fY0:=Value;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;
  fReady:=false;
end;

function TLabordeOblic.GetNom_ParamReal(index : integer) : string;
begin
  case index of
    0 : result:='k0';
    1 : result:='X0';
    2 : result:='Y0';
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

function TLabordeOblic.GetNbInteger : integer;
begin
  result:=0;
end;

function TLabordeOblic.GetParamInteger(index : integer) : Integer;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TLabordeOblic.SetParamInteger(index : integer; Value : Integer);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TLabordeOblic.GetNom_ParamInteger(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TLabordeOblic.GetNbAngle : integer;
begin
  result:=3;
end;

function TLabordeOblic.GetParamAngle(index : integer) : string;
begin
  case index of
    0 : result:=fLanda0;
    1 : result:=fPhi0;
    2 : result:=fTheta0;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;  
end;

procedure TLabordeOblic.SetParamAngle(index : integer; Value : string);
begin
  case index of
    0 : begin
      try
        fLanda0:=Value;
        fLanda0real:=StrToAngle(Value);
        fReady:=false;
      except
        fValid:=false;
        Raise;
      end;
    end;
    1 : begin
      try
        fPhi0:=Value;
        fPhi0real:=StrToAngle(Value);
        fReady:=false;
      except
        fValid:=false;
        Raise;
      end;
    end;
    2 : begin
      try
        fTheta0:=Value;
        fTheta0real:=StrToAngle(Value);
        fReady:=false;
      except
        fValid:=false;
        Raise;
      end;
    end;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;
end;

function TLabordeOblic.GetNom_ParamAngle(index : integer) : string;
begin
  case index of
    0 : result:='Landa0';
    1 : result:='Phi0';
    2 : result:='Theta0';
  else
    begin
      result:='';
      raise ERangeError.Create('Demande de paramtre hors-limites');
    end;
  end;
end;

function TLabordeOblic.GetNbBoolean : integer;
begin
  result:=0;
end;

function TLabordeOblic.GetParamBoolean(index : integer) : Boolean;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TLabordeOblic.SetParamBoolean(index : integer; Value : Boolean);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TLabordeOblic.GetNom_ParamBoolean(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TLabordeOblic.Prepare;
var
  e2 : real;
  sph0 : real;
begin
  e2:=sqr(fDatum.Ellipsoide.e);
  sph0:=sin(fPhi0Real);

  r:=fDatum.Ellipsoide.a*sqrt(1-e2)/(1-e2*sqr(sph0))*fk0;
  phisl := arctan(tan(fphi0Real)*sqrt((1.0-e2)/(1.0-e2*sph0*sph0)));
  alpha:=sqrt(1.0+e2/(1.-e2)*power(cos(fphi0Real),4.0));
  fchi0 := sqrt((1.0+sph0)/(1.0-sph0)*power((1.0-fDatum.Ellipsoide.e*sph0)/(1.0+fDatum.Ellipsoide.e*sph0),fDatum.Ellipsoide.e));
  acon := (1.0 - cos(2.0*fTheta0Real))/12.0;
  bcon := sin(2.0*fTheta0Real)/12.0;

  fReady:=true;
end;

procedure TLabordeOblic.VersGrille(Lat, Long : real; var X, Y : real);
var
  sph, fchi, fac, phil, dl, bfac, x_ptt, cl, y_ptt, v3, v4 : real;
begin
  if not fReady then
    Prepare;

  sph := sin(Lat);
  fchi := sqrt((1.0+sph)/(1.0-sph)*power((1.0-fDatum.Ellipsoide.e*sph)/
                         (1.0+fDatum.Ellipsoide.e*sph),fDatum.Ellipsoide.e));
  fac := power((fchi/fchi0),alpha)*tan(pi/4.0+phisl/2.0);
  phil := 2.0*arctan(fac)-pi/2.0;
  dl := alpha*(Long-fLanda0Real);
  bfac := cos(phil)*sin(dl);

  // This is the Gauss-Schreiber Transverse Mercator for unit sphere x,y := X,Y

  x_ptt:=0.5*ln((1.0+bfac)/(1.0-bfac));
  cl := cos(dl);
  y_ptt := arctan(tan(phil)/cl) - phisl;
  if (cl < 0.0) then y := y+ pi;

// Third order complex rotation with false eastings and false northings

  v3 := x_ptt*(3.0*y_ptt*y_ptt-x_ptt*x_ptt);
  v4 := y_ptt*(y_ptt*y_ptt-3.0*x_ptt*x_ptt);
  x := fX0+r*(x_ptt+acon*v3+bcon*v4);
  y := fY0+r*(y_ptt-bcon*v3+acon*v4);


end;

procedure TLabordeOblic.VersLatLong(X, Y : real; var Lat, Long : real);
var
  xpr,ypr,v5,v6,v7,v8,v9,v10,v11 : real;
  dx,dy : real;
  coshx,sinhx,fac1,dphi : real;
  x_ptt,y_ptt,v3,v4,fac : real;
  phil,fchi,sph : real;
begin
  if not fReady then
    Prepare;

  xpr := (X-fX0)/r;
  ypr := (Y-fY0)/r;
  x_ptt := xpr;
  y_ptt := ypr;
  repeat
     v3 := x_ptt*(3.0*y_ptt*y_ptt-x_ptt*x_ptt);
     v4 := y_ptt*(y_ptt*y_ptt-3.0*x_ptt*x_ptt);
     v5 := 3.0*(y_ptt*y_ptt-x_ptt*x_ptt);
     v6 := 6.0*x_ptt*y_ptt;
     v7 := y_ptt-bcon*v3+acon*v4-ypr;
     v8 := x_ptt+acon*v3+bcon*v4-xpr;
     v9 := 1.0+acon*v5-bcon*v6;
     v10 := bcon*v5+acon*v6;
     v11 := v9*v9+v10*v10;
     dx := -(v8*v9-v7*v10)/v11;
     dy := -(v7*v9+v8*v10)/v11;
     x_ptt := x_ptt + dx;
     y_ptt := y_ptt + dy;
  until abs(dx)+abs(dy)<0.00001/r;

//   Here, x,y are Gauss-Schreiber Transverse Mercator coordinates
  fac := exp(x_ptt);
  coshx := (fac+1.0/fac)/2.0;
  sinhx := (fac-1.0/fac)/2.0;
  fac1 := sin(y_ptt+phisl)/coshx;
  phil := arctan(fac1/sqrt(abs(1.0-fac1*fac1)));
  Long := fLanda0Real+arctan(sinhx/cos(y_ptt+phisl))/alpha;
  fchi := fchi0*power((tan(pi/4.0+phil/2.0)/tan(pi/4.0+phisl/2.0)),(1.0/alpha));
  Lat := phil;
  repeat
     sph := sin(Lat);
     dphi := 2.0 * arctan(fchi*power((1.0+fDatum.Ellipsoide.e*sph)/
         (1.0-fDatum.Ellipsoide.e*sph),(fDatum.Ellipsoide.e/2.0)))-pi/2.0-Lat;
     Lat := Lat+ dphi;
  until abs(dphi) < 1.0e-15;
end;

end.

