unit mercator_multi;

interface

uses
  multiple, transmercator;

type

// projection mercator avec des fuseaux
// un numro de fuseau >0 correspond  l'hmisphre Nord
// un numro de fuseau <0 correspond au fuseau de nombre oppos
// dans l'hmisphre Sud

  TMultipleMercator = class(TMultiple)
  private
    Nord, Sud : array of TTransverseMercator;
    fFuseauOrigine : string; // longitude du centre du fuseau numro 1
    fFuseauOrigineReal : real;  // valeur en radian de la longitude prcdente
    fPas : string; // pas entre un fuseau et le suivant, valeur ngative si la numrotation augmente vers l'ouest
    fPasReal : real;  // valeur en radian du pas prcdent
    fk0 : real; // facteur d'chelle au point d'origine
    fX0, fY0Nord, fY0Sud : real; // coordonnes en projection du point d'origine
    function GetFuseau(fuseau : integer) : TTransverseMercator;
  protected
    // real
    function GetNbReal : integer; override;
    function GetParamReal(index : integer) : real; override;
    procedure SetParamReal(index : integer; Value : real); override;
    function GetNom_ParamReal(index : integer) : string; override;
    // integer
    function GetNbInteger : integer; override;
    function GetParamInteger(index : integer) : Integer; override;
    procedure SetParamInteger(index : integer; Value : Integer); override;
    function GetNom_ParamInteger(index : integer) : string; override;
    // Angle
    function GetNbAngle : integer; override;
    function GetParamAngle(index : integer) : string; override;
    procedure SetParamAngle(index : integer; Value : string); override;
    function GetNom_ParamAngle(index : integer) : string; override;
    // Boolean
    function GetNbBoolean : integer; override;
    function GetParamBoolean(index : integer) : Boolean; override;
    procedure SetParamBoolean(index : integer; Value : Boolean); override;
    function GetNom_ParamBoolean(index : integer) : string; override;

    function GetNature : string; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure VersGrille(Lat, Long : real; var X, Y : real; var fuseau : integer); override;
    procedure VersLatLong(X, Y : real; fuseau : integer; var Lat, Long : real); override;
  end;

implementation

uses
  SysUtils,
  outils, math;

constructor TMultipleMercator.Create;
begin
  inherited Create;
end;

destructor TMultipleMercator.Destroy;
var
  I : integer;
begin
  for I:=1 to High(Nord) do
    TTransverseMercator(Nord[I]).Free;
  for I:=1 to High(Sud) do
    TTransverseMercator(Sud[I]).Free;
  inherited Destroy;
end;


function TMultipleMercator.GetNature : string;
begin
  result:=tmMercatorMultiple;
end;

function TMultipleMercator.GetNbReal : integer;
begin
  result:=4;
end;

function TMultipleMercator.GetParamReal(index : integer) : real;
begin
  case index of
    0 : result:=fk0;
    1 : result:=fX0;
    2 : result:=fY0Nord;
    3 : result:=fY0Sud;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

procedure TMultipleMercator.SetParamReal(index : integer; Value : real);
begin
  case Index of
    0 : fk0:=Value;
    1 : fX0:=Value;
    2 : fY0Nord:=Value;
    3 : fY0Sud:=Value;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;
end;

function TMultipleMercator.GetNom_ParamReal(index : integer) : string;
begin
  case index of
    0 : result:='k0';
    1 : result:='X0';
    2 : result:='Y0Nord';
    3 : result:='Y0Sud';
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

function TMultipleMercator.GetNbInteger : integer;
begin
  result:=0;
end;

function TMultipleMercator.GetParamInteger(index : integer) : Integer;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleMercator.SetParamInteger(index : integer; Value : Integer);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleMercator.GetNom_ParamInteger(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TMultipleMercator.GetNbAngle : integer;
begin
  result:=2;
end;

function TMultipleMercator.GetParamAngle(index : integer) : string;
begin
  case index of
    0 : result:=fFuseauOrigine;
    1 : result:=fPas;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

procedure TMultipleMercator.SetParamAngle(index : integer; Value : string);
begin
  case index of
    0 : begin
      try
        fFuseauOrigine:=Value;
        fFuseauOriginereal:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
    1 : begin
      try
        fPas:=Value;
        fPasreal:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;  
end;

function TMultipleMercator.GetNom_ParamAngle(index : integer) : string;
begin
  case index of
    0 : result:='LandaFuseau0';
    1 : result:='Pas';
  else
    begin
      result:='';
      raise ERangeError.Create('Demande de paramtre hors-limites');
    end;
  end;
end;

function TMultipleMercator.GetNbBoolean : integer;
begin
  result:=0;
end;

function TMultipleMercator.GetParamBoolean(index : integer) : Boolean;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleMercator.SetParamBoolean(index : integer; Value : Boolean);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleMercator.GetNom_ParamBoolean(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;


function TMultipleMercator.GetFuseau(fuseau : integer) : TTransverseMercator;
begin
  if fuseau>0 then
  begin
    if fuseau>high(Nord) then
      SetLength(Nord,fuseau+1);
    if Nord[fuseau]=nil then
    begin
      result:=TTransverseMercator.Create;
      result.Datum:=Datum;
      result.ParamReal[0]:=fk0;
      result.ParamReal[1]:=fX0;
      result.ParamReal[2]:=fY0Nord;
      result.ParamAngle[0]:=AngleToStrDegreSexa((fuseau-1)*fPasReal+fFuseauOrigineReal);
      result.ParamAngle[1]:='0';
      result.Valid:=true;
      Nord[fuseau]:=result;
    end;
    result:=Nord[fuseau];
  end
  else begin
    if -fuseau>high(Sud) then
      SetLength(Sud,(-fuseau)+1);
    if Sud[-fuseau]=nil then
    begin
      result:=TTransverseMercator.Create;
      result.Datum:=Datum;
      result.ParamReal[0]:=fk0;
      result.ParamReal[1]:=fX0;
      result.ParamReal[2]:=fY0Sud;
      result.ParamAngle[0]:=AngleToStrGrade((-fuseau-1)*fPasReal+fFuseauOrigineReal);
      result.ParamAngle[1]:='0';
      result.Valid:=true;
      Sud[-fuseau]:=result;
    end;
    result:=Sud[-fuseau];
  end;
end;

procedure TMultipleMercator.VersGrille(Lat, Long : real; var X, Y : real; var fuseau : integer);
var
  Alpha : real;
begin
  // on ramne la longitude en valeurs positives
  Alpha:=Long-fFuseauOrigineReal+fPasReal/2;
  Alpha:=Alpha*sign(fPasReal);
  while Alpha<0 do
    Alpha:=Alpha+2*Pi;
  while Alpha>2*Pi do
    Alpha:=Alpha-2*Pi;
  fuseau:=trunc(Alpha/abs(fPasReal))+1;
  if Lat<0 then
    fuseau:=-fuseau;
  GetFuseau(fuseau).VersGrille(Lat,Long,X, Y);
end;

procedure TMultipleMercator.VersLatLong(X, Y : real; fuseau : integer; var Lat, Long : real);
begin
  GetFuseau(fuseau).VersLatLong(X,Y,Lat, Long);
end;

end.
