unit mtm_multi;

// Note : il serait judicieux de raliser un anctre commun  TMultipleMTM
// et TMultipleUTM

interface

uses
  multiple, mtm;

type

// projection MTM avec prise en compte des fuseaux
// un numro de fuseau >0 correspond  l'hmisphre Nord
// un numro de fuseau <0 correspond au fuseau de nombre oppos
// dans l'hmisphre Sud

  TMultipleMtm = class(TMultiple)
  private
    Nord, Sud : array[1..120] of TMTM;
    function GetFuseau(fuseau : integer) : TMTM;
  protected
    // real
    function GetNbReal : integer; override;
    function GetParamReal(index : integer) : real; override;
    procedure SetParamReal(index : integer; Value : real); override;
    function GetNom_ParamReal(index : integer) : string; override;
    // integer
    function GetNbInteger : integer; override;
    function GetParamInteger(index : integer) : Integer; override;
    procedure SetParamInteger(index : integer; Value : Integer); override;
    function GetNom_ParamInteger(index : integer) : string; override;
    // Angle
    function GetNbAngle : integer; override;
    function GetParamAngle(index : integer) : string; override;
    procedure SetParamAngle(index : integer; Value : string); override;
    function GetNom_ParamAngle(index : integer) : string; override;
    // Boolean
    function GetNbBoolean : integer; override;
    function GetParamBoolean(index : integer) : Boolean; override;
    procedure SetParamBoolean(index : integer; Value : Boolean); override;
    function GetNom_ParamBoolean(index : integer) : string; override;

    function GetNature : string; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure VersGrille(Lat, Long : real; var X, Y : real; var fuseau : integer); override;
    procedure VersLatLong(X, Y : real; fuseau : integer; var Lat, Long : real); override;
  end;

implementation

uses
  SysUtils,
  outils;

constructor TMultipleMtm.Create;
var
  I : integer;
begin
  inherited Create;
  for I:=1 to 120 do
  begin
    Nord[I]:=nil;
    Sud[I]:=nil;
  end;
end;

destructor TMultipleMtm.Destroy;
var
  I : integer;
begin
  for I:=1 to 120 do
  begin
    TMTM(Nord[I]).Free;
    TMTM(Sud[I]).Free;
  end;
  inherited Destroy;
end;


function TMultipleMtm.GetNature : string;
begin
  result:=tmMTM;
end;

function TMultipleMtm.GetNbReal : integer;
begin
  result:=0;
end;

function TMultipleMtm.GetParamReal(index : integer) : real;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleMtm.SetParamReal(index : integer; Value : real);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleMtm.GetNom_ParamReal(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TMultipleMtm.GetNbInteger : integer;
begin
  result:=0;
end;

function TMultipleMtm.GetParamInteger(index : integer) : Integer;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleMtm.SetParamInteger(index : integer; Value : Integer);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleMtm.GetNom_ParamInteger(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TMultipleMtm.GetNbAngle : integer;
begin
  result:=0;
end;

function TMultipleMtm.GetParamAngle(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleMtm.SetParamAngle(index : integer; Value : string);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleMtm.GetNom_ParamAngle(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TMultipleMtm.GetNbBoolean : integer;
begin
  result:=0;
end;

function TMultipleMtm.GetParamBoolean(index : integer) : Boolean;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleMtm.SetParamBoolean(index : integer; Value : Boolean);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleMtm.GetNom_ParamBoolean(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;


function TMultipleMtm.GetFuseau(fuseau : integer) : TMTM;
begin
  if fuseau>0 then
  begin
    if Nord[fuseau]=nil then
    begin
      result:=TMTM.Create;
      result.Datum:=Datum;
      result.ParamInteger[0]:=fuseau;
      result.ParamBoolean[0]:=false;
      Nord[fuseau]:=result;
    end;
    result:=Nord[fuseau];
  end
  else begin
    if Sud[-fuseau]=nil then
    begin
      result:=TMTM.Create;
      result.Datum:=Datum;
      result.ParamInteger[0]:=-fuseau;
      result.ParamBoolean[0]:=true;
      Sud[-fuseau]:=result;
    end;
    result:=Sud[-fuseau];
  end;
end;

procedure TMultipleMtm.VersGrille(Lat, Long : real; var X, Y : real; var fuseau : integer);
var
  LongFus : real;
begin
  LongFus:=-Long; // les fuseaux augmentent vers l'ouest
  LongFus:=LongFus-51/180*Pi; // le premier fuseau commence  -51
  while LongFus<0 do // on remet en positif
    LongFus:=LongFus+2*Pi;
  fuseau:=trunc(LongFus/Pi*60);
  fuseau:=fuseau mod 120 +1;
  if Lat<0 then
    fuseau:=-fuseau;
  GetFuseau(fuseau).VersGrille(Lat,Long,X, Y);
end;

procedure TMultipleMtm.VersLatLong(X, Y : real; fuseau : integer; var Lat, Long : real);
begin
  GetFuseau(fuseau).VersLatLong(X,Y,Lat, Long);
end;

end.
