unit obliquelaborde;

// cas gnrale de la projection Gauss-Laborde Oblique

// documentation de rfrence
//
// Trait des projections
// des
// cartes gographiques
//  l'usage
// des cartographes et des godsiens
// L. Driencourt et J. Laborde
// Quatrime fascicule
// Thorie de la reprsentation conforme
// Emploi des projections rigoureusement conformes en godsie
// J. Laborde
// Paris, Ed. Hermann, 1932
//
// mlang avec
//
// IGN
// Projection cartographique Gauss-Laborde
// Notes techniques NT/G 73
//
// http://www.ign.fr/fr/TVCHARG/SGN/DOCTRANSFO/NTG_73.pdf

interface

uses
  projection;

type

  TLabordeOblic = class(TProjection)
  protected
    fn1 : real; // exposant de la projection ellipsode-sphre
    fn2 : real; // rayon de la sphre intermdiaire
    fc : real; // constante de la projection
    fLandaC : string; // longitude de l'origine par rapport au mridien d'origine du Datum
    fLandaCReal : real; // valeur en radian de la longitude prcdente
    fPhiC : string; //latitude sur la sphre du centre de la projection
    fPhiCReal : real; // valeur en radian de la latitude prcdente
    fTheta0 : string; // obliquit de l'indicatrice
    fTheta0Real : real; // valeur en radian de l'obliquit prcdente
    fXs, fYs : real; // coordonnes en projection du ple (false Est et false Nord)
    // real
    function GetNbReal : integer; override;
    function GetParamReal(index : integer) : real; override;
    procedure SetParamReal(index : integer; Value : real); override;
    function GetNom_ParamReal(index : integer) : string; override;
    // integer
    function GetNbInteger : integer; override;
    function GetParamInteger(index : integer) : Integer; override;
    procedure SetParamInteger(index : integer; Value : Integer); override;
    function GetNom_ParamInteger(index : integer) : string; override;
    // Angle
    function GetNbAngle : integer; override;
    function GetParamAngle(index : integer) : string; override;
    procedure SetParamAngle(index : integer; Value : string); override;
    function GetNom_ParamAngle(index : integer) : string; override;
    // Boolean
    function GetNbBoolean : integer; override;
    function GetParamBoolean(index : integer) : Boolean; override;
    procedure SetParamBoolean(index : integer; Value : Boolean); override;
    function GetNom_ParamBoolean(index : integer) : string; override;

    function GetNature : string; override;
    procedure Remonte(n1, n2, c, Xs, Ys : real; Landa0, Phi0, Theta0 : string); // remonte les paramtres calculs par les descandants dans l'anctre TLabordeOblic
  public
    procedure VersGrille(Lat, Long : real; var X, Y : real); override;
    procedure VersLatLong(X, Y : real; var Lat, Long : real); override;
  end;

implementation

uses
  SysUtils,
  outils, ign, ign_laborde;


function TLabordeOblic.GetNature : string;
begin
  result:=tpGaussLabordeOblique;
end;

function TLabordeOblic.GetNbReal : integer;
begin
  result:=5;
end;

function TLabordeOblic.GetParamReal(index : integer) : real;
begin
  case index of
    0 : result:=fn1;
    1 : result:=fn2;
    2 : result:=fc;
    3 : result:=fXs;
    4 : result:=fYs;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

procedure TLabordeOblic.SetParamReal(index : integer; Value : real);
begin
  case Index of
    0 : fn1:=Value;
    1 : fn2:=Value;
    2 : fc:=Value;
    3 : fXs:=Value;
    4 : fYs:=Value;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;
end;

function TLabordeOblic.GetNom_ParamReal(index : integer) : string;
begin
  case index of
    0 : result:='n1';
    1 : result:='n2';
    2 : result:='c';
    3 : result:='X0';
    4 : result:='Y0';
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

function TLabordeOblic.GetNbInteger : integer;
begin
  result:=0;
end;

function TLabordeOblic.GetParamInteger(index : integer) : Integer;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TLabordeOblic.SetParamInteger(index : integer; Value : Integer);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TLabordeOblic.GetNom_ParamInteger(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TLabordeOblic.GetNbAngle : integer;
begin
  result:=3;
end;

function TLabordeOblic.GetParamAngle(index : integer) : string;
begin
  case index of
    0 : result:=fLandaC;
    1 : result:=fPhiC;
    2 : result:=fTheta0;
  else
    raise ERangeError.Create('Demande de paramtre hors-limites');
  end;
end;

procedure TLabordeOblic.SetParamAngle(index : integer; Value : string);
begin
  case index of
    0 : begin
      try
        fLandaC:=Value;
        fLandaCreal:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
    1 : begin
      try
        fPhiC:=Value;
        fPhiCreal:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
    2 : begin
      try
        fTheta0:=Value;
        fTheta0real:=StrToAngle(Value);
      except
        fValid:=false;
        Raise;
      end;
    end;
  else
    begin
      fValid:=false;
      raise ERangeError.Create('Ecriture de paramtre hors-limites');
    end;
  end;
end;

function TLabordeOblic.GetNom_ParamAngle(index : integer) : string;
begin
  case index of
    0 : result:='LandaC';
    1 : result:='PhiC';
    2 : result:='Theta0';
  else
    begin
      result:='';
      raise ERangeError.Create('Demande de paramtre hors-limites');
    end;
  end;
end;

function TLabordeOblic.GetNbBoolean : integer;
begin
  result:=0;
end;

function TLabordeOblic.GetParamBoolean(index : integer) : Boolean;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TLabordeOblic.SetParamBoolean(index : integer; Value : Boolean);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TLabordeOblic.GetNom_ParamBoolean(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;




procedure TLabordeOblic.VersGrille(Lat, Long : real; var X, Y : real);
begin
  alg0034_laborde(Long,Lat,Datum.Ellipsoide.e,fLandaCReal, fPhiCReal, fTheta0Real,
          fn1, fn2, fXs, fYs, fc, X, Y);
end;

procedure TLabordeOblic.VersLatLong(X, Y : real; var Lat, Long : real);
begin
  alg0035(X, Y, Datum.Ellipsoide.e, fLandaCReal, fn1, fn2, fXs, fYs, fc, Long, Lat);
end;

procedure TLabordeOblic.Remonte(n1, n2, c, Xs, Ys : real; Landa0, Phi0, Theta0 : string);
// remonte les paramtres calculs par les descandants dans l'anctre TLabordeOblic
begin
  fn1:=n1;
  fn2:=n2;
  fc:=c;
  fXs:=Xs;
  fYs:=Ys;
  fLandaC:=Landa0;
  fLandaCReal:=StrToAngle(fLandaC);
  fPhiC:=Phi0;
  fPhiCReal:=StrToAngle(fPhiC);
  fTheta0:=Theta0;
  fTheta0Real:=StrToAngle(fTheta0);
end;

end.
