unit utm_multi;

// Note : il serait judicieux de raliser un anctre commun  TMultipleMTM
// et TMultipleUTM

interface

uses
  multiple, utm;

type

// projection UTM avec prise en compte des fuseaux
// un numro de fuseau >0 correspond  l'hmisphre Nord
// un numro de fuseau <0 correspond au fuseau de nombre oppos
// dans l'hmisphre Sud

  TMultipleUtm = class(TMultiple)
  private
    Nord, Sud : array[1..60] of TUTM;
    function GetFuseau(fuseau : integer) : TUTM;
  protected
    // real
    function GetNbReal : integer; override;
    function GetParamReal(index : integer) : real; override;
    procedure SetParamReal(index : integer; Value : real); override;
    function GetNom_ParamReal(index : integer) : string; override;
    // integer
    function GetNbInteger : integer; override;
    function GetParamInteger(index : integer) : Integer; override;
    procedure SetParamInteger(index : integer; Value : Integer); override;
    function GetNom_ParamInteger(index : integer) : string; override;
    // Angle
    function GetNbAngle : integer; override;
    function GetParamAngle(index : integer) : string; override;
    procedure SetParamAngle(index : integer; Value : string); override;
    function GetNom_ParamAngle(index : integer) : string; override;
    // Boolean
    function GetNbBoolean : integer; override;
    function GetParamBoolean(index : integer) : Boolean; override;
    procedure SetParamBoolean(index : integer; Value : Boolean); override;
    function GetNom_ParamBoolean(index : integer) : string; override;

    function GetNature : string; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure VersGrille(Lat, Long : real; var X, Y : real; var fuseau : integer); override;
    procedure VersLatLong(X, Y : real; fuseau : integer; var Lat, Long : real); override;
  end;

implementation

uses
  SysUtils,
  outils;

constructor TMultipleUtm.Create;
var
  I : integer;
begin
  inherited Create;
  for I:=1 to 60 do
  begin
    Nord[I]:=nil;
    Sud[I]:=nil;
  end;
end;

destructor TMultipleUtm.Destroy;
var
  I : integer;
begin
  for I:=1 to 60 do
  begin
    TUTM(Nord[I]).Free;
    TUTM(Sud[I]).Free;
  end;
  inherited Destroy;
end;


function TMultipleUtm.GetNature : string;
begin
  result:=tmUTM;
end;

function TMultipleUtm.GetNbReal : integer;
begin
  result:=0;
end;

function TMultipleUtm.GetParamReal(index : integer) : real;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleUtm.SetParamReal(index : integer; Value : real);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleUtm.GetNom_ParamReal(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TMultipleUtm.GetNbInteger : integer;
begin
  result:=0;
end;

function TMultipleUtm.GetParamInteger(index : integer) : Integer;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleUtm.SetParamInteger(index : integer; Value : Integer);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleUtm.GetNom_ParamInteger(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TMultipleUtm.GetNbAngle : integer;
begin
  result:=0;
end;

function TMultipleUtm.GetParamAngle(index : integer) : string;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleUtm.SetParamAngle(index : integer; Value : string);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleUtm.GetNom_ParamAngle(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

function TMultipleUtm.GetNbBoolean : integer;
begin
  result:=0;
end;

function TMultipleUtm.GetParamBoolean(index : integer) : Boolean;
begin
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;

procedure TMultipleUtm.SetParamBoolean(index : integer; Value : Boolean);
begin
  fValid:=false;
  raise ERangeError.Create('Ecriture de paramtre hors-limites');
end;

function TMultipleUtm.GetNom_ParamBoolean(index : integer) : string;
begin
  result:='';
  raise ERangeError.Create('Demande de paramtre hors-limites');
end;


function TMultipleUtm.GetFuseau(fuseau : integer) : TUTM;
begin
  if fuseau>0 then
  begin
    if Nord[fuseau]=nil then
    begin
      result:=TUTM.Create;
      result.Datum:=Datum;
      result.ParamInteger[0]:=fuseau;
      result.ParamBoolean[0]:=false;
      Nord[fuseau]:=result;
    end;
    result:=Nord[fuseau];
  end
  else begin
    if Sud[-fuseau]=nil then
    begin
      result:=TUTM.Create;
      result.Datum:=Datum;
      result.ParamInteger[0]:=-fuseau;
      result.ParamBoolean[0]:=true;
      Sud[-fuseau]:=result;
    end;
    result:=Sud[-fuseau];
  end;
end;

procedure TMultipleUtm.VersGrille(Lat, Long : real; var X, Y : real; var fuseau : integer);
begin
  // on ramne la longitude en valeurs positives
  while Long<0 do
    Long:=Long+2*Pi;
  fuseau:=trunc((Long+Pi)/Pi*30);
  fuseau:=fuseau mod 60 +1;
  if Lat<0 then
    fuseau:=-fuseau;
  GetFuseau(fuseau).VersGrille(Lat,Long,X, Y);
end;

procedure TMultipleUtm.VersLatLong(X, Y : real; fuseau : integer; var Lat, Long : real);
begin
  GetFuseau(fuseau).VersLatLong(X,Y,Lat, Long);
end;

end.
